/// <reference types="node" />
/// <reference types="node" />
import type { Compiler, Dev } from "@rspack/core";
import type { Logger } from "./logger";
import type { Socket } from "net";
import type { FSWatcher, WatchOptions } from "chokidar";
import type { WebSocketServer, ClientConnection } from "./ws";
import type { Application } from "express";
import type { Server } from "http";
import type { ResolvedDev } from "./config";
declare type Host = "local-ip" | "local-ipv4" | "local-ipv6" | string;
declare type Port = number | string | "auto";
export declare class RspackDevServer {
    compiler: Compiler;
    options: ResolvedDev;
    logger: Logger;
    staticWatchers: FSWatcher[];
    sockets: Socket[];
    app: Application;
    server: Server;
    private listeners;
    private currentHash;
    private middleware;
    webSocketServer: WebSocketServer | undefined;
    constructor(compiler: Compiler);
    normalizeOptions(dev?: Dev): ResolvedDev;
    rewriteCompilerOptions(): void;
    addAdditionEntires(): void;
    static isAbsoluteURL(URL: string): boolean;
    static findIp(gateway: string): string | undefined;
    static internalIP(family: "v6" | "v4"): Promise<string | undefined>;
    static internalIPSync(family: "v6" | "v4"): Promise<string | undefined>;
    static getHostname(hostname?: Host): Promise<string>;
    static getFreePort(port: Port, host: string): Promise<string | number>;
    static findCacheDir(): string;
    private getCompilerOptions;
    sendMessage(clients: ClientConnection[], type: string, data?: any, params?: any): void;
    watchFiles(watchPath: string | string[], watchOptions?: WatchOptions): void;
    invalidate(callback?: () => void): void;
    start(): Promise<void>;
    startCallback(callback?: (err?: Error) => void): void;
    stopCallback(callback?: (err?: Error) => void): void;
    listen(port: Port, hostname: string, fn: (err?: Error) => void): void;
    close(callback?: (err?: Error) => void): void;
    stop(): Promise<void>;
    private setupApp;
    private setupWatchStaticFiles;
    private setupDevMiddleware;
    private createWebsocketServer;
    private setupMiddlewares;
    private createServer;
    private setupHooks;
}
export {};
//# sourceMappingURL=server.d.ts.map