"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RspackDevServer = void 0;
const dev_middleware_1 = require("@rspack/dev-middleware");
const fs_1 = __importDefault(require("fs"));
const chokidar_1 = __importDefault(require("chokidar"));
const http_1 = __importDefault(require("http"));
const logger_1 = require("./logger");
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const express_1 = __importDefault(require("express"));
const dev_middleware_2 = __importDefault(require("@rspack/dev-middleware"));
const ws_1 = require("./ws");
const config_1 = require("./config");
// copy from webpack-dev-server
class RspackDevServer {
    constructor(compiler) {
        this.compiler = compiler;
        this.logger = (0, logger_1.createLogger)("rspack-dev-server");
        this.staticWatchers = [];
        this.listeners = [];
        this.sockets = [];
        this.currentHash = "";
        this.options = this.normalizeOptions(compiler.options.devServer);
        this.rewriteCompilerOptions();
        this.addAdditionEntires();
    }
    normalizeOptions(dev = {}) {
        return (0, config_1.resolveDevOptions)(dev, this.compiler.options);
    }
    rewriteCompilerOptions() {
        var _a, _b;
        this.compiler.options.devServer = this.options;
        if (!this.compiler.options.builtins.react) {
            this.compiler.options.builtins.react = {};
        }
        this.compiler.options.builtins.react.development =
            (_a = this.compiler.options.builtins.react.development) !== null && _a !== void 0 ? _a : true;
        if (this.options.hot) {
            this.compiler.options.builtins.react.refresh =
                (_b = this.compiler.options.builtins.react.refresh) !== null && _b !== void 0 ? _b : true;
        }
        else if (this.compiler.options.builtins.react.refresh) {
            this.logger.warn("[Builtins] react.refresh need react.development and devServer.hot enabled.");
        }
    }
    addAdditionEntires() {
        var _a;
        const entries = [];
        if (this.options.hot) {
            const hotUpdateEntryPath = require.resolve("@rspack/dev-client/devServer");
            entries.push(hotUpdateEntryPath);
            if ((_a = this.compiler.options.builtins.react) === null || _a === void 0 ? void 0 : _a.refresh) {
                const reactRefreshEntryPath = require.resolve("@rspack/dev-client/react-refresh");
                entries.push(reactRefreshEntryPath);
            }
        }
        const devClientEntryPath = require.resolve("@rspack/dev-client");
        entries.push(devClientEntryPath);
        for (const key in this.compiler.options.entry) {
            this.compiler.options.entry[key].import.unshift(...entries);
        }
    }
    static isAbsoluteURL(URL) {
        return webpack_dev_server_1.default.isAbsoluteURL(URL);
    }
    static findIp(gateway) {
        return webpack_dev_server_1.default.findIp(gateway);
    }
    static async internalIP(family) {
        return webpack_dev_server_1.default.internalIP(family);
    }
    static async internalIPSync(family) {
        return webpack_dev_server_1.default.internalIPSync(family);
    }
    static async getHostname(hostname) {
        return webpack_dev_server_1.default.getHostname(hostname);
    }
    static async getFreePort(port, host) {
        return webpack_dev_server_1.default.getFreePort(port, host);
    }
    static findCacheDir() {
        // TODO: we need remove the `webpack-dev-server` tag in WebpackDevServer;
        return "";
    }
    getCompilerOptions() {
        return this.compiler.options;
    }
    sendMessage(clients, type, data, params) {
        for (const client of clients) {
            if (client.readyState === 1) {
                client.send(JSON.stringify({ type, data, params }));
            }
        }
    }
    watchFiles(watchPath, watchOptions) {
        const watcher = chokidar_1.default.watch(watchPath, watchOptions);
        // disabling refreshing on changing the content
        if (this.options.liveReload) {
            // TODO: remove this after we had memory filesystem
            if (this.options.hot) {
                return;
            }
            watcher.on("change", item => {
                if (this.webSocketServer) {
                    this.sendMessage(this.webSocketServer.clients, "static-changed", item);
                }
            });
        }
        this.staticWatchers.push(watcher);
    }
    invalidate(callback = () => { }) {
        if (this.middleware) {
            this.middleware.invalidate(callback);
        }
    }
    async start() {
        this.setupHooks();
        this.setupApp();
        this.createServer();
        this.setupWatchStaticFiles();
        this.createWebsocketServer();
        this.setupDevMiddleware();
        this.setupMiddlewares();
        const host = await RspackDevServer.getHostname(this.options.host);
        const port = await RspackDevServer.getFreePort(this.options.port, host);
        this.options.port = port;
        await new Promise(resolve => this.server.listen({
            port,
            host
        }, () => {
            this.logger.info(`Loopback: http://localhost:${port}`);
            let internalIPv4 = webpack_dev_server_1.default.internalIPSync("v4");
            this.logger.info(`Your Network (IPV4) http://${internalIPv4}:${port}`);
            resolve({});
        }));
    }
    startCallback(callback) {
        throw new Error("Method not implemented.");
    }
    stopCallback(callback) {
        throw new Error("Method not implemented.");
    }
    listen(port, hostname, fn) {
        throw new Error("Method not implemented.");
    }
    close(callback) {
        throw new Error("Method not implemented.");
    }
    async stop() {
        this.compiler.close(() => { });
        await Promise.all(this.staticWatchers.map(watcher => watcher.close()));
        this.staticWatchers = [];
        if (this.middleware) {
            await new Promise((resolve, reject) => {
                this.middleware.close((error) => {
                    if (error) {
                        reject(error);
                        return;
                    }
                    resolve(undefined);
                });
            });
        }
        this.middleware = null;
        if (this.server) {
            this.server.close();
        }
        if (this.webSocketServer) {
            await new Promise(resolve => {
                this.webSocketServer.implementation.close(() => {
                    this.webSocketServer = null;
                    resolve(void 0);
                });
                for (const client of this.webSocketServer.clients)
                    client.terminate();
            });
        }
    }
    setupApp() {
        this.app = (0, express_1.default)();
    }
    setupWatchStaticFiles() {
        if (this.options.static.watch === false) {
            return;
        }
        this.watchFiles(this.options.static.directory, this.options.static.watch);
    }
    setupDevMiddleware() {
        // @ts-ignored
        this.middleware = (0, dev_middleware_2.default)(this.compiler, this.options.devMiddleware);
    }
    createWebsocketServer() {
        if (this.options.webSocketServer !== false) {
            this.webSocketServer = (0, ws_1.createWebsocketServer)(this);
        }
    }
    setupMiddlewares() {
        const options = this.options;
        const middlewares = [];
        middlewares.push({
            name: "rdm",
            middleware: this.middleware
        });
        if (this.compiler.options.experiments.lazyCompilation) {
            middlewares.push({
                middleware: (req, res, next) => {
                    if (req.url.indexOf("/lazy-compilation-web/") > -1) {
                        const path = req.url.replace("/lazy-compilation-web/", "");
                        if (fs_1.default.existsSync(path)) {
                            this.compiler.rebuild(new Set([path]), new Set(), error => {
                                if (error) {
                                    throw error;
                                }
                                res.write("");
                                res.end();
                                console.log("lazy compiler success");
                            });
                        }
                    }
                }
            });
        }
        // Todo Add options
        const connectHistoryApiFallback = require("connect-history-api-fallback");
        middlewares.push({
            name: "[connect-history-api-fallback]",
            middleware: connectHistoryApiFallback({
                verbose: true,
                logger: console.log.bind(console)
            })
        });
        /**
         * supports three kinds of proxy configuration
         * {context: 'xxxx', target: 'yyy}
         * {['xxx']: { target: 'yyy}}
         * [{context: 'xxx',target:'yyy'}, {context: 'aaa', target: 'zzzz'}]
         */
        if (typeof options.proxy !== "undefined") {
            const { createProxyMiddleware } = require("http-proxy-middleware");
            function getProxyMiddleware(proxyConfig) {
                if (proxyConfig.target) {
                    const context = proxyConfig.context || proxyConfig.path;
                    return createProxyMiddleware(context, proxyConfig);
                }
                if (proxyConfig.router) {
                    return createProxyMiddleware(proxyConfig);
                }
            }
            if (!Array.isArray(options.proxy)) {
                if (Object.prototype.hasOwnProperty.call(options.proxy, "target") ||
                    Object.prototype.hasOwnProperty.call(options.proxy, "router")) {
                    options.proxy = [options.proxy];
                }
                else {
                    options.proxy = Object.keys(options.proxy).map(context => {
                        let proxyOptions;
                        // For backwards compatibility reasons.
                        const correctedContext = context
                            .replace(/^\*$/, "**")
                            .replace(/\/\*$/, "");
                        if (typeof ( /** @type {ProxyConfigMap} */options.proxy[context]) ===
                            "string") {
                            proxyOptions = {
                                context: correctedContext,
                                target: 
                                /** @type {ProxyConfigMap} */
                                options.proxy[context]
                            };
                        }
                        else {
                            proxyOptions = {
                                // @ts-ignore
                                ... /** @type {ProxyConfigMap} */options.proxy[context]
                            };
                            proxyOptions.context = correctedContext;
                        }
                        return proxyOptions;
                    });
                }
            }
            options.proxy.forEach(proxyConfig => {
                const handler = async (req, res, next) => {
                    let proxyMiddleware = getProxyMiddleware(proxyConfig);
                    const isByPassFuncDefined = typeof proxyConfig.bypass === "function";
                    const bypassUrl = isByPassFuncDefined
                        ? await proxyConfig.bypass(req, res, proxyConfig)
                        : null;
                    if (typeof bypassUrl === "boolean") {
                        req.url = null;
                        next();
                    }
                    else if (typeof bypassUrl === "string") {
                        req.url = bypassUrl;
                    }
                    else if (proxyMiddleware) {
                        return proxyMiddleware(req, res, next);
                    }
                    else {
                        next();
                    }
                };
                middlewares.push({
                    name: "http-proxy-middleware",
                    middleware: handler
                });
                middlewares.push({
                    name: "http-proxy-middleware-error-handler",
                    middleware: (error, req, res, next) => handler(req, res, next)
                });
            });
        }
        const publicPath = this.compiler.options.output.publicPath === "auto"
            ? ""
            : this.compiler.options.output.publicPath;
        middlewares.push({
            name: "rspack-memory-assets",
            path: publicPath,
            middleware: (0, dev_middleware_1.getRspackMemoryAssets)(this.compiler)
        });
        middlewares.push({
            name: "express-static",
            path: publicPath,
            middleware: express_1.default.static(this.options.static.directory)
        });
        middlewares.forEach(m => {
            if (m.path) {
                this.app.use(m.path, m.middleware);
            }
            else {
                this.app.use(m.middleware);
            }
        });
    }
    createServer() {
        this.server = http_1.default.createServer(this.app);
    }
    setupHooks() {
        this.compiler.hooks.done.tap("dev-server", stats => {
            // send Message
            if (this.webSocketServer) {
                this.sendMessage(this.webSocketServer.clients, "ok"); // TODO: send hash
            }
        });
    }
}
exports.RspackDevServer = RspackDevServer;
//# sourceMappingURL=server.js.map